/*
 * \file Inputs.c
 * \brief: implementation of the Inputs class, derived from DataSet class
 */

/*Headers: {{{*/
#ifdef HAVE_CONFIG_H
	#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "./Input.h"
#include "./Inputs.h"
#include "../../shared/shared.h"

using namespace std;
/*}}}*/

/*Object constructors and destructor*/
Inputs::Inputs(){/*{{{*/
	return;
}
/*}}}*/
Inputs::~Inputs(){/*{{{*/
	return;
}
/*}}}*/

/*Object management*/
int  Inputs::AddInput(Input* in_input){/*{{{*/

	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 
	_assert_(in_input); 
	int enum_name = in_input->InstanceEnum();
	printf("Adding %s to inputs\n",EnumToStringx(enum_name));
	#ifdef _ISSM_DEBUG_
	if(!IsInputEnum(enum_name)) _error_("Enum \""<<EnumToStringx(enum_name)<<"\" is not placed properly in EnumDefinitions.h");
	 #endif

	/*Go through current inputs*/
	int size = this->Size();
	for(int i=0;i<size;i++){
		Input* input=xDynamicCast<Input*>(this->objects[i]);

		/*If it's already there, remove it and replace it with in_input*/
		if(input->InstanceEnum()==enum_name){
			delete input;
			this->objects[i] = in_input;
			return 1;
		}
	}

	/*It is new, add input to the dataset*/
	this->AddObject(in_input);

	return 1;
}
/*}}}*/
void  Inputs::ChangeEnum(int oldenumtype,int newenumtype){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Delete input if it already exists*/
	this->DeleteInput(newenumtype);
	_assert_(IsInputEnum(oldenumtype));
	_assert_(IsInputEnum(newenumtype));

	/*Go through current inputs*/
	int size = this->Size();
	for(int i=0;i<size;i++){
		Input* input=xDynamicCast<Input*>(this->objects[i]);

		/*If it's already there, remove it and replace it with in_input*/
		if(input->InstanceEnum()==oldenumtype){
			input->ChangeEnum(newenumtype);
			return;
		}
	}
}/*}}}*/
void Inputs::Configure(Parameters* parameters){/*{{{*/

	vector<Object*>::iterator object;
	Input* input=NULL;

	for ( object=objects.begin() ; object < objects.end(); object++ ){

		input=xDynamicCast<Input*>(*object);
		input->Configure(parameters);

	}

}
/*}}}*/
int  Inputs::DeleteInput(int enum_type){/*{{{*/

	vector<Object*>::iterator object;
	Input* input=NULL;
	bool found = false;

	for ( object=objects.begin() ; object < objects.end(); object++ ){
		input=xDynamicCast<Input*>(*object);
		if (input->InstanceEnum()==enum_type){
			found = true;
			break;
		}
	}
	if(found) this->DeleteObject(input);
	return 1;

}
/*}}}*/
void  Inputs::DuplicateInput(int original_enum,int new_enum){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Make a copy of the original input: */
	Input* original=this->GetInput(original_enum);
	if(!original)_error_("could not find input with enum: " << EnumToStringx(original_enum)); 
	Input* copy=xDynamicCast<Input*>(original->copy());

	/*Change copy enum to reinitialized_enum: */
	copy->ChangeEnum(new_enum);

	/*Add copy into inputs, it will wipe off the one already there: */
	this->AddInput(xDynamicCast<Input*>(copy));
}
/*}}}*/
Input* Inputs::GetInput(int enum_name){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	vector<Object*>::iterator object;
	Input* input=NULL;

	for ( object=objects.begin() ; object < objects.end(); object++ ){
		input=xDynamicCast<Input*>(*object);
		if (input->InstanceEnum()==enum_name) return input;
	}

	return NULL;
}
/*}}}*/
void Inputs::GetInputAverage(IssmDouble* pvalue,int enum_type){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enum_type);
	if (!input) _error_("could not find input with enum type " << enum_type << " (" << EnumToStringx(enum_type) << ")");

	/*Ok, we have an input if we made it here, request the input to return the value: */
	input->GetInputAverage(pvalue);

}
/*}}}*/
void Inputs::GetInputValue(bool* pvalue,int enum_type){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enum_type);
	if (!input) _error_("could not find input with enum type " << enum_type << " (" << EnumToStringx(enum_type) << ")");

	/*Ok, we have an input if we made it here, request the input to return the value: */
	input->GetInputValue(pvalue);

}
/*}}}*/
void Inputs::GetInputValue(int* pvalue,int enum_type){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enum_type);
	if (!input) _error_("could not find input with enum type " << enum_type << " (" << EnumToStringx(enum_type) << ")");

	/*Ok, we have an input if we made it here, request the input to return the value: */
	input->GetInputValue(pvalue);

}/*}}}*/
void Inputs::GetInputValue(IssmDouble* pvalue,int enum_type){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enum_type);
	if (!input) _error_("could not find input with enum type " << enum_type << " (" << EnumToStringx(enum_type) << ")");

	/*Ok, we have an input if we made it here, request the input to return the value: */
	input->GetInputValue(pvalue);

}/*}}}*/
IssmDouble Inputs::Max(int enumtype){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enumtype);
	if (!input) _error_("could not find input with enum " << EnumToStringx(enumtype));

	/*Return output*/
	return input->Max();

}/*}}}*/
IssmDouble Inputs::MaxAbs(int enumtype){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enumtype);
	if (!input) _error_("could not find input with enum " << EnumToStringx(enumtype));

	/*Return output*/
	return input->MaxAbs();

}/*}}}*/
IssmDouble Inputs::Min(int enumtype){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enumtype);
	if (!input) _error_("could not find input with enum " << EnumToStringx(enumtype));

	/*Return output*/
	return input->Min();

}/*}}}*/
IssmDouble Inputs::MinAbs(int enumtype){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Find input in current dataset*/
	Input* input=this->GetInput(enumtype);
	if (!input) _error_("could not find input with enum " << EnumToStringx(enumtype));

	/*Return output*/
	return input->MinAbs();

}/*}}}*/
Inputs* Inputs::SpawnSegInputs(int index1,int index2){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Intermediary*/
	vector<Object*>::iterator object;
	Input* inputin=NULL;
	Input* inputout=NULL;

	/*Output*/
	Inputs* newinputs=new Inputs();

	/*Go through inputs and call Spawn function*/
	for ( object=objects.begin() ; object < objects.end(); object++ ){

		/*Create new input*/
		inputin=xDynamicCast<Input*>(*object);
		inputout=inputin->SpawnSegInput(index1,index2);

		/*Add input to new inputs*/
		newinputs->AddObject(inputout);
	}

	/*Assign output pointer*/
	return newinputs;
}
/*}}}*/
Inputs* Inputs::SpawnTriaInputs(int index1,int index2,int index3){/*{{{*/
	printf("-------------- file: Inputs.cpp line: %i\n",__LINE__); 

	/*Intermediary*/
	vector<Object*>::iterator object;
	Input* inputin=NULL;
	Input* inputout=NULL;

	/*Output*/
	Inputs* newinputs=new Inputs();

	/*Go through inputs and call Spawn function*/
	for(object=objects.begin() ; object < objects.end(); object++ ){

		/*Create new input*/
		inputin=xDynamicCast<Input*>(*object);
		inputout=inputin->SpawnTriaInput(index1,index2,index3);

		/*Add input to new inputs*/
		newinputs->AddObject(inputout);
	}

	/*Assign output pointer*/
	return newinputs;
}
/*}}}*/
